'use client';
import * as React from 'react';
import { format } from 'date-fns';
import { Calendar as CalendarIcon, ChevronDownIcon } from 'lucide-react';
import { Button, type buttonVariants } from '@/components/ui/button';
import { type VariantProps } from 'class-variance-authority';
import { Calendar } from '@/components/ui/calendar';
import { Input } from '@/components/ui/input';
import { Label } from '@/components/ui/label';
import {
  Popover,
  PopoverContent,
  PopoverTrigger,
} from '@/components/ui/popover';

type BaseDatePickerProps = {
  value?: Date;
  onValueChange?: (date: Date | undefined) => void;
  disabled?: boolean;
  placeholder?: string;
  dateFormat?: string;
  popoverContentProps?: React.ComponentProps<typeof PopoverContent>;
};

type SimpleDatePickerProps = BaseDatePickerProps & {
  variant: 'simple';
  label?: string;
  labelProps?: React.ComponentProps<typeof Label>;
  buttonProps?: React.ComponentProps<typeof Button> &
    VariantProps<typeof buttonVariants>;
  iconProps?: React.ComponentProps<typeof ChevronDownIcon>;
};

type WithInputDatePickerProps = BaseDatePickerProps & {
  variant: 'with-input';
  label?: string;
  labelProps?: React.ComponentProps<typeof Label>;
  inputProps?: React.ComponentProps<typeof Input>;
  buttonProps?: React.ComponentProps<typeof Button> &
    VariantProps<typeof buttonVariants>;
  iconProps?: React.ComponentProps<typeof CalendarIcon>;
  allowTextInput?: boolean;
};

type DateTimePickerProps = BaseDatePickerProps & {
  variant: 'date-time';
  dateLabel?: string;
  timeLabel?: string;
  dateLabelProps?: React.ComponentProps<typeof Label>;
  timeLabelProps?: React.ComponentProps<typeof Label>;
  dateButtonProps?: React.ComponentProps<typeof Button> &
    VariantProps<typeof buttonVariants>;
  timeInputProps?: React.ComponentProps<typeof Input>;
  iconProps?: React.ComponentProps<typeof ChevronDownIcon>;
  defaultTime?: string;
  onTimeChange?: (time: string) => void;
  containerClassName?: string;
};

type DatePickerProps =
  | SimpleDatePickerProps
  | WithInputDatePickerProps
  | DateTimePickerProps;

const formatDate = (date: Date | undefined, dateFormat?: string) => {
  if (!date) return '';
  if (dateFormat) return format(date, dateFormat);
  return date.toLocaleDateString('en-US', {
    day: '2-digit',
    month: 'long',
    year: 'numeric',
  });
};

const isValidDate = (date: Date | undefined) => {
  if (!date) return false;
  return !isNaN(date.getTime());
};

const SimpleDatePicker = ({
  value,
  onValueChange,
  disabled = false,
  placeholder = 'Select date',
  dateFormat,
  label,
  labelProps,
  buttonProps = {
    className: 'w-48 justify-between font-normal',
    variant: 'outline',
  },
  iconProps = {
    className: 'ml-2 h-4 w-4 shrink-0 opacity-50',
  },
  popoverContentProps = {
    className: 'w-auto overflow-hidden p-0',
    align: 'start',
  },
}: Omit<SimpleDatePickerProps, 'variant'>) => {
  const [open, setOpen] = React.useState(false);

  return (
    <div className='flex flex-col gap-3'>
      {label && (
        <Label htmlFor='date' className='px-1' {...labelProps}>
          {label}
        </Label>
      )}
      <Popover open={open} onOpenChange={setOpen}>
        <PopoverTrigger asChild>
          <Button id='date' disabled={disabled} {...buttonProps}>
            {value ? formatDate(value, dateFormat) : placeholder}
            <ChevronDownIcon {...iconProps} />
          </Button>
        </PopoverTrigger>
        <PopoverContent {...popoverContentProps}>
          <Calendar
            mode='single'
            captionLayout='dropdown'
            selected={value}
            onSelect={onValueChange}
            disabled={disabled}
          />
        </PopoverContent>
      </Popover>
    </div>
  );
};

const WithInputDatePicker = ({
  value,
  onValueChange,
  disabled = false,
  placeholder = 'June 01, 2025',
  dateFormat,
  label,
  labelProps,
  inputProps,
  buttonProps = {
    className: 'absolute top-1/2 right-2 size-6 -translate-y-1/2',
    variant: 'ghost',
  },
  iconProps = {
    className: 'size-3.5',
  },
  popoverContentProps = {
    className: 'w-auto overflow-hidden p-0',
    align: 'end',
    alignOffset: -8,
    sideOffset: 10,
  },
  allowTextInput = true,
}: Omit<WithInputDatePickerProps, 'variant'>) => {
  const [open, setOpen] = React.useState(false);
  const [month, setMonth] = React.useState<Date | undefined>(value);
  const [inputValue, setInputValue] = React.useState(
    formatDate(value, dateFormat),
  );

  React.useEffect(() => {
    setInputValue(formatDate(value, dateFormat));
    setMonth(value);
  }, [value, dateFormat]);

  const handleInputChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const newValue = e.target.value;
    setInputValue(newValue);

    if (allowTextInput) {
      const parsedDate = new Date(newValue);
      if (isValidDate(parsedDate)) {
        onValueChange?.(parsedDate);
        setMonth(parsedDate);
      }
    }
  };

  const handleDateSelect = (selectedDate: Date | undefined) => {
    onValueChange?.(selectedDate);
    setInputValue(formatDate(selectedDate, dateFormat));
    setOpen(false);
  };

  return (
    <div className='flex flex-col gap-3'>
      {label && (
        <Label htmlFor='date' className='px-1' {...labelProps}>
          {label}
        </Label>
      )}
      <div className='relative flex gap-2'>
        <Input
          id='date'
          value={inputValue}
          placeholder={placeholder}
          className='bg-background pr-10'
          disabled={disabled}
          onChange={handleInputChange}
          onKeyDown={(e) => {
            if (e.key === 'ArrowDown') {
              e.preventDefault();
              setOpen(true);
            }
          }}
          {...inputProps}
        />
        <Popover open={open} onOpenChange={setOpen}>
          <PopoverTrigger asChild>
            <Button id='date-picker' disabled={disabled} {...buttonProps}>
              <CalendarIcon {...iconProps} />
              <span className='sr-only'>Select date</span>
            </Button>
          </PopoverTrigger>
          <PopoverContent {...popoverContentProps}>
            <Calendar
              mode='single'
              captionLayout='dropdown'
              selected={value}
              month={month}
              onMonthChange={setMonth}
              onSelect={handleDateSelect}
              disabled={disabled}
            />
          </PopoverContent>
        </Popover>
      </div>
    </div>
  );
};

const DateTimePicker = ({
  value,
  onValueChange,
  disabled = false,
  placeholder = 'Select date',
  dateFormat,
  dateLabel = 'Date',
  timeLabel = 'Time',
  dateLabelProps,
  timeLabelProps,
  dateButtonProps = {
    className: 'w-32 justify-between font-normal',
    variant: 'outline',
  },
  timeInputProps,
  iconProps = {
    className: 'ml-2 h-4 w-4 shrink-0 opacity-50',
  },
  popoverContentProps = {
    className: 'w-auto overflow-hidden p-0',
    align: 'start',
  },
  defaultTime = '10:30:00',
  onTimeChange,
  containerClassName = 'flex gap-4',
}: Omit<DateTimePickerProps, 'variant'>) => {
  const [open, setOpen] = React.useState(false);

  const handleDateSelect = (selectedDate: Date | undefined) => {
    onValueChange?.(selectedDate);
    setOpen(false);
  };

  return (
    <div className={containerClassName}>
      <div className='flex flex-col gap-3'>
        <Label htmlFor='date-picker' className='px-1' {...dateLabelProps}>
          {dateLabel}
        </Label>
        <Popover open={open} onOpenChange={setOpen}>
          <PopoverTrigger asChild>
            <Button id='date-picker' disabled={disabled} {...dateButtonProps}>
              {value ? formatDate(value, dateFormat) : placeholder}
              <ChevronDownIcon {...iconProps} />
            </Button>
          </PopoverTrigger>
          <PopoverContent {...popoverContentProps}>
            <Calendar
              mode='single'
              captionLayout='dropdown'
              selected={value}
              onSelect={handleDateSelect}
              disabled={disabled}
            />
          </PopoverContent>
        </Popover>
      </div>
      <div className='flex flex-col gap-3'>
        <Label htmlFor='time-picker' className='px-1' {...timeLabelProps}>
          {timeLabel}
        </Label>
        <Input
          type='time'
          id='time-picker'
          step='1'
          defaultValue={defaultTime}
          disabled={disabled}
          className='bg-background appearance-none [&::-webkit-calendar-picker-indicator]:hidden [&::-webkit-calendar-picker-indicator]:appearance-none'
          onChange={(e) => onTimeChange?.(e.target.value)}
          {...timeInputProps}
        />
      </div>
    </div>
  );
};

export const DatePicker = (props: DatePickerProps) => {
  switch (props.variant) {
    case 'simple':
      return <SimpleDatePicker {...props} />;
    case 'with-input':
      return <WithInputDatePicker {...props} />;
    case 'date-time':
      return <DateTimePicker {...props} />;
    default:
      return null;
  }
};
