import { type SupabaseClient, type Profile } from '@/utils/supabase';
import { getSignedUrl } from '@/lib/queries';

const signUp = (client: SupabaseClient, formData: FormData) => {
  const full_name = formData.get('name') as string;
  const email = formData.get('email') as string;
  const password = formData.get('password') as string;
  const origin = process.env.NEXT_PUBLIC_SITE_URL!;
  return client.auth.signUp({
    email,
    password,
    options: {
      emailRedirectTo: `${origin}/auth/callback`,
      data: {
        full_name,
        email,
        provider: 'email',
      }
    }
  });
};

const signIn = (client: SupabaseClient, formData: FormData) => {
  const email = formData.get('email') as string;
  const password = formData.get('password') as string;
  return client.auth.signInWithPassword({ email, password });
};

const signInWithMicrosoft = (client: SupabaseClient) => {
  const origin = process.env.NEXT_PUBLIC_SITE_URL!;
  return client.auth.signInWithOAuth({
    provider: 'azure',
    options: {
      scopes: 'openid profile email offline_access',
      redirectTo: `${origin}/auth/callback?redirect_to=/auth/success`,
    },
  });
};

const signInWithApple = (client: SupabaseClient) => {
  const origin = process.env.NEXT_PUBLIC_SITE_URL!;
  return client.auth.signInWithOAuth({
    provider: 'apple',
    options: {
      scopes: 'openid profile email offline_access',
      redirectTo: `${origin}/auth/callback?redirect_to=/auth/success`,
    },
  });
};

const forgotPassword = (client: SupabaseClient, formData: FormData) => {
  const email = formData.get('email') as string;
  const origin = process.env.NEXT_PUBLIC_SITE_URL!;
  return client.auth.resetPasswordForEmail(email, {
    redirectTo: `${origin}/auth/callback?redirect_to=/profile`,
  });
};

const resetPassword = (client: SupabaseClient, formData: FormData) => {
  const password = formData.get('password') as string;
  return client.auth.updateUser({ password });
};

const signOut = (client: SupabaseClient) => {
  return client.auth.signOut();
}

const getCurrentUser = (client: SupabaseClient) => {
  return client.auth.getUser();
};

const getProfile = (client: SupabaseClient, userId: string) => {
  return client
    .from(`profiles`)
    .select(`*`)
    .eq(`id`, userId)
    .single();
};

const getAvatar = (client: SupabaseClient, avatarUrl: string) => {
  return getSignedUrl({
    client,
    bucket: 'avatars',
    path: avatarUrl,
    seconds: 3600,
    transform: {
      width: 128,
      height: 128,
    },
  });
};

const updateProfile = (
  client: SupabaseClient,
  userId: string,
  updates: Partial<Profile>,
) => {
  return client
    .from(`profiles`)
    .update(updates)
    .eq(`id`, userId)
    .select()
    .single();
};

export {
  forgotPassword,
  getCurrentUser,
  getProfile,
  getAvatar,
  resetPassword,
  signIn,
  signInWithApple,
  signInWithMicrosoft,
  signOut,
  signUp,
  updateProfile
};
